﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Drawing;

using System.IO;
using System.Diagnostics;
using System.Threading;

//AForge Libraries
using AForge.Video;
using AForge.Video.DirectShow;
using AForge.Video.FFMPEG;
using AForge.Video.VFW;

using Microsoft.Win32;//used for getting sleep mode event

namespace Clarus_Med_Scope_Viewer
{
    //Event Argument class for custom event to trigger on video recording start/stop
    public class VideoRecordingStatusArgs : EventArgs
    {
        public enum captureEventType {VideoStarted,VideoStopped,PictureCaptured}
        public captureEventType CapEventType { get; set; }
        public VideoRecordingStatusArgs(captureEventType VideoEvent)
        {
            CapEventType = VideoEvent;
        }
    }

    //Event Argument class for custom event to handle timer when an error occures
    public class TimerErrorArgs : EventArgs
    {
        public enum errorStatusType { Ok, Loading, Error }
        public errorStatusType ErrorStatus { get; set; }
        public TimerErrorArgs(errorStatusType ErrorStat)
        {
            ErrorStatus = ErrorStat;
        }
    }


    public class ScopeViewController
    {
        private Logging _debugLogger;
        private bool _debugLoggingEnabled = false;

        private FilterInfoCollection VideoDevices;
        public VideoCaptureDevice VideoDevice;
        public string VidDeviceName = "";
        private VideoCapabilities[] _videoCapabilities;
        private AForge.Controls.VideoSourcePlayer _videoSourcePlayer;
        delegate void _vidPlayerDelegate();


        private string _folderPath = "";
        private string _strStudyName = "";

        DateTime _videoCapStartTime;
        private bool _captureArmed = false;
        public object _caputreArmedLock = new object();

        public double VidWidthFactor;
        public bool _videoRecording = false;

        private bool _pause = false;
        public object _pauseLock = new object();


        private bool _videoInitialized = false;
        public object _videoInitializedLock = new object();

        public enum CaptureModeType { Picture, Video }
        private CaptureModeType _captureMode = CaptureModeType.Picture;

        //object used to for locking to access cross thread data
        public object _threadLock = new object();

        //object for encoding video
        private VideoFileWriter vFileWriter = new VideoFileWriter();

        //timer for checking if video has come back online
        private System.Threading.Timer _vidRecoveryTimer;
        private bool recoveryStarted = false;

        //check for zero bytes read
        private System.Threading.Timer _checkForCaptureTimer;
        private Stopwatch _stopWatchError;
        private TimerErrorArgs.errorStatusType _errorStatus = TimerErrorArgs.errorStatusType.Ok;
        private long _lastBytesRead = 0;

        private int _numOfStartFrames = 4;//v0.0.8 number of frames to skip before starting capture
        private int _currStartFrame = 0;//v0.0.8 current count of start frames to check against _numOfStartFrames

        public string _camMonikerStringExclusions = "";//v0.0.9 used to identify correct camera for remote shutter feature
        public string _currentCamMoniker = "";

        private bool _remCaptAllowed = false;//v0.0.9
        private object _remCaptAllowedLock = new object();//v0.0.9

        private bool _capturingPicture = false;//flag to determine if a picture captures is active
        private object _capturingPictureLock = new object();//object to lock for cross thread operations

        private bool _captureEventActive = false;
        private object _captureEventActiveLock = new object();

        public event EventHandler<VideoRecordingStatusArgs> VideoStatusChanged;
        public void RaiseVideoStatusEvent(VideoRecordingStatusArgs.captureEventType VideoCaptureEventType)
        {
            VideoStatusChanged?.Invoke(null, new VideoRecordingStatusArgs(VideoCaptureEventType));
        }
        public event EventHandler<TimerErrorArgs> TimerErrorChanged;
        public void RaiseTimerErrorEvent(TimerErrorArgs.errorStatusType ErrorStatType)
        {
            TimerErrorChanged?.Invoke(null, new TimerErrorArgs(ErrorStatType));
        }

        private object _currBitmapLock = new object();
        private Bitmap _currentBitmap;

        public CaptureModeType CaptureMode
        {
            get
            {
                lock (_threadLock)
                    return _captureMode;
            }
            set
            {
                lock (_threadLock)
                    _captureMode = value;
            }
        }
        public bool VideoRecordingNow
        {
            get
            {
                lock (_threadLock)
                    return _videoRecording;
            }
        }

        public string FolderPath
        {
            get
            {
                lock (_threadLock)
                    return _folderPath;
            }
            set
            {
                lock (_threadLock)
                    _folderPath = value;
            }

        }
        public string StudyName
        {
            get
            {
                lock (_threadLock)
                    return _strStudyName;
            }
            set
            {
                lock (_threadLock)
                    _strStudyName = value;
            }

        }

        //v0.0.9
        public bool CaptureEventActive
        {
            get
            {
                lock (_captureEventActiveLock)
                    return _captureEventActive;
            }
            set
            {
                lock (_captureEventActiveLock)
                    _captureEventActive = value;
            }
        }

        public string VideoDeviceName { get; set; }
        public int VideoDeviceHeight { get; set; }
        public int VideoDeviceWidth { get; set; }

        //constructor overload to use debug logging
        public ScopeViewController(AForge.Controls.VideoSourcePlayer VideoSourcePlayer, Logging DebugLogger, string camMonikerString)
        {
            _camMonikerStringExclusions = camMonikerString;
            _debugLogger = DebugLogger;
            _debugLoggingEnabled = true;

            initialize(VideoSourcePlayer);
        }

        //constructor, pass in the video player
        public ScopeViewController(AForge.Controls.VideoSourcePlayer VideoSourcePlayer, string camMonikerString)
        {
            _camMonikerStringExclusions = camMonikerString;
            initialize(VideoSourcePlayer);
        }

        private void initialize(AForge.Controls.VideoSourcePlayer VideoSourcePlayer)
        {
            if (_debugLoggingEnabled)
            { _debugLogger.addLog("Initializing Scope View Controller"); }

            //v0.0.9 subscribe to the power changed event
            SystemEvents.PowerModeChanged += SystemEvents_PowerModeChanged;

            //reference VideoSourcePlayerObect from UI
            _videoSourcePlayer = VideoSourcePlayer;

            //setup video recovery timer
            _vidRecoveryTimer = new System.Threading.Timer(VidRecovCallback, null, Timeout.Infinite, Timeout.Infinite);

            //setup timer to poll for capture trigger
            _checkForCaptureTimer = new System.Threading.Timer(checkForCaptureCallback, null, 100, 100);

            //_videoWriteTimer = new System.Threading.Timer(videoWriteCallback, null, 30, 30);

            _stopWatchError = new Stopwatch();
        }


        //v0.0.9 handle computer going into sleep mode
        void SystemEvents_PowerModeChanged(object sender, PowerModeChangedEventArgs e)
        {
            if (e.Mode == PowerModes.Suspend)
            {
                //https://stackoverflow.com/questions/3425203/how-do-i-gracefully-handle-hibernate-sleep-modes-in-a-winforms-application/3496566
                if (_debugLoggingEnabled)
                { _debugLogger.addLog("Computer going into sleep mode..."); }

                stopCapture();
                lock(_pauseLock)
                {
                    _pause = true;
                }
                _videoSourcePlayer.Stop();

            }
            if (e.Mode == PowerModes.Resume)
            {
                if (_debugLoggingEnabled)
                { _debugLogger.addLog("Computer power mode resume..."); }
                _videoSourcePlayer.Start();
                lock (_pauseLock)
                {
                    _pause = false;
                }
            }
        }

        private void VidRecovCallback(object state)
        { recoverVideo(); }

        private void checkForCaptureCallback(object state)
        { checkForCapture(); }

        //private void videoWriteCallback(object state)
        //{
        //    writeVideoFrame();
        //}


        //private void writeVideoFrame()
        //{
        //    _videoWriteTimer.Change(Timeout.Infinite, Timeout.Infinite);
        //    int captureTimeDelay = 25;

            

        //    lock (_currBitmapLock)
        //    {
        //        try
        //        {
        //            //_aviVideoStream.AddFrame(videoBitmap);
        //            Bitmap newBm = _videoSourcePlayer.GetCurrentVideoFrame();
        //            _aviWriter.AddFrame(newBm);
        //            //vFileWriter.WriteVideoFrame(videoBitmap, timeSpan);
        //        }
        //        catch (Exception ex)
        //        {
        //            //log errors?
        //            if (_debugLoggingEnabled)
        //            { _debugLogger.addLog(ex.Message); }
        //        }
        //    }
            


        //    captureTimeDelay = 1000 / _frameRate;

        //    _videoWriteTimer.Change(captureTimeDelay, captureTimeDelay);
        //}

        private void checkForCapture()
        {
            
            _checkForCaptureTimer.Change(Timeout.Infinite, Timeout.Infinite);
            long currentBytesRead = 1;
            //bool captureDetected = false;
            bool videoCaptureDetected = false;
            bool videoInitialized = false;
            lock(_videoInitializedLock)
                videoInitialized = _videoInitialized;
            bool pause = false;
            lock (_pauseLock)
                pause = _pause;
            bool captureArmed = false;
            lock (_caputreArmedLock)
                captureArmed = _captureArmed;

            bool remoteCaptureAllowed = false;
            lock(_remCaptAllowedLock)
            { remoteCaptureAllowed = _remCaptAllowed; }

            
            if (videoInitialized == true && pause == false && remoteCaptureAllowed == true)
            {
                lock (_threadLock)
                { currentBytesRead = VideoDevice.BytesReceived; }

                long averageBytesRead = (currentBytesRead + _lastBytesRead) / 2;
                _lastBytesRead = currentBytesRead;

                if (averageBytesRead == 0)
                {
                    if (captureArmed == false)
                    {
                            
                        //captureDetected = true;
                        if (_videoRecording == false)
                        {
                            startCapture();
                            videoCaptureDetected = true;
                            if (_debugLoggingEnabled)
                            { _debugLogger.addLog("Bytes Read Zero - starting capture"); }

                            //lock(_caputreArmedLock)
                            //    _captureArmed = true;
                        }
                        else
                        {
                            //_videoInitialized = false;
                            if (_debugLoggingEnabled)
                            { _debugLogger.addLog("Bytes Read Zero - stopping capture"); }

                            stopCapture();
                            
                        }
                    }
                    else //capture is armed
                    {
                        if(_stopWatchError.IsRunning)
                        {
                            if (_stopWatchError.ElapsedMilliseconds > 500 && _stopWatchError.ElapsedMilliseconds < 8000 && _errorStatus != TimerErrorArgs.errorStatusType.Loading)
                            { //show loading symbol
                                _errorStatus = TimerErrorArgs.errorStatusType.Loading;
                                RaiseTimerErrorEvent(TimerErrorArgs.errorStatusType.Loading);
                            }
                            else if (_stopWatchError.ElapsedMilliseconds >= 8000 && _errorStatus != TimerErrorArgs.errorStatusType.Error)
                            { //show error message
                                RaiseTimerErrorEvent(TimerErrorArgs.errorStatusType.Error);
                            }
                        }
                        else
                        {
                            _stopWatchError = Stopwatch.StartNew();
                        }
                    }
                }
            }


            if (videoCaptureDetected)
            {
                _checkForCaptureTimer.Change(500, 500);
            }
            else
            {
                _checkForCaptureTimer.Change(50, 50);
            }

        }

        
        private void recoverVideo()
        {
            bool pause = false;
            lock (_pauseLock)
                pause = _pause;

            if (pause == false)
            {
                lock (_threadLock)
                {
                    _vidRecoveryTimer.Change(Timeout.Infinite, Timeout.Infinite);
                    this._videoSourcePlayer.BeginInvoke(new Action(
                                () =>
                                {
                                    if (_videoSourcePlayer.IsRunning == false)
                                    {
                                        if (_debugLoggingEnabled)
                                        { _debugLogger.addLog("Recover Attempt --> Video Sorce Not Running"); }


                                    //_videoSourcePlayer.SignalToStop();
                                    //_videoSourcePlayer.WaitForStop();
                                    //setupVideoDevice();
                                    VideoDevice.Start();
                                    //_videoSourcePlayer.Start();

                                }
                                    else
                                    {
                                        if (_debugLoggingEnabled)
                                        { _debugLogger.addLog("Recover Attempt --> Video Source IS Running"); }


                                    //_videoSourcePlayer.SignalToStop();
                                    //_videoSourcePlayer.WaitForStop();
                                    //setupVideoDevice();
                                    VideoDevice.Start();
                                    //_videoSourcePlayer.Start();

                                }
                                }));
                    if (recoveryStarted)
                        _vidRecoveryTimer.Change(500, 500);

                }

            }//if pause == false

        }

        public bool changeVideoDevice(VideoCaptureDevice NewVideoDevice, string NewVideoDeviceName, 
            int NewVideoDeviceHeight, int NewVideoDeviceWidth, string monikerRemoteExclusions)
        {
            _camMonikerStringExclusions = monikerRemoteExclusions;//0.0.9 update remote captue exlusions list on each video device change

            bool videoChangedOk = false;
            //first stop the videoDevice
            lock (_videoInitializedLock)
                _videoInitialized = false;

            lock (_threadLock)
            {
                //_videoInitialized = false; //moved above
                VidDeviceName = NewVideoDeviceName;
                try
                {
                    if (_videoRecording == false)
                    {
                        lock(_pauseLock)
                            _pause = true;
                        
                        Close();
                        videoChangedOk = loadVideoDeviceOnStartup(NewVideoDeviceName, NewVideoDeviceHeight, NewVideoDeviceWidth);
                        
                        lock(_pauseLock)
                            _pause = false;

                    }
                    videoChangedOk = true;
                }
                catch
                {
                    videoChangedOk = false;
                }
            }
            return videoChangedOk;
        }

        public bool startCapture()
        {
            bool pause = false;
            lock (_pauseLock)
                pause = _pause;

            bool captureArmed = true;
            lock (_caputreArmedLock)
                captureArmed = _captureArmed;

            bool startedOk = false;

            if (pause == false && captureArmed == false)
            {
                lock (_threadLock)
                {
                    _currStartFrame = 0; //v0.0.8

                    if (_videoRecording == false)
                    {
                        if (_captureMode == CaptureModeType.Picture)
                        {
                            try
                            {
                                //v0.6 
                                lock (_caputreArmedLock)
                                    _captureArmed = true;

                                startedOk = true;
                            }
                            catch
                            {
                                startedOk = false;
                            }
                        }
                        else if (_captureMode == CaptureModeType.Video)
                        {
                            try
                            {
                                string fileName = DateTime.Now.ToString("yyyyMMddHHmmssmmFFF") + "_" + _strStudyName + ".avi";

                                vFileWriter.Open(_folderPath + fileName, VideoDevice.VideoResolution.FrameSize.Width, VideoDevice.VideoResolution.FrameSize.Height, 30);

                                _videoCapStartTime = DateTime.Now;
                                _videoRecording = true;
                                startedOk = true;
                                RaiseVideoStatusEvent(VideoRecordingStatusArgs.captureEventType.VideoStarted);
                            }
                            catch
                            {
                                startedOk = false;
                            }
                        }
                    }
                    else
                    {
                        startedOk = false; //can't start because its already started
                                           //_videoRecording = false;
                    }
                }
            }

            if (_debugLoggingEnabled)
            { _debugLogger.addLog("Capture Started OK = " + startedOk); }

            return startedOk;
        }

        private void capturePicture(Bitmap picture)
        {
            if (_debugLoggingEnabled)
            { _debugLogger.addLog("Picture Capture Started"); }

            bool capPicActive = false;
            lock (_capturingPictureLock)
            { capPicActive = _capturingPicture; }

            Bitmap picCopy = picture.Clone(new Rectangle(0, 0, picture.Width, picture.Height), picture.PixelFormat);

            if (capPicActive == false)
            {
                lock (_capturingPictureLock)
                { _capturingPicture = true; }

                try
                {

                    string fileName = DateTime.Now.ToString("yyyyMMddHHmmssFFF") + "_" + _strStudyName + ".bmp";

                    if (_debugLoggingEnabled)
                    { _debugLogger.addLog("Saving Picture --> " + fileName); }

                    picCopy.Save(_folderPath + fileName);
                    if (_debugLoggingEnabled)
                    { _debugLogger.addLog(fileName + " picture save complete --> " + fileName); }


                    //v0.0.9
                    bool captureEventActive = false;
                    lock (_captureEventActiveLock)
                        captureEventActive = _captureEventActive;

                    if (captureEventActive == false)
                        RaiseVideoStatusEvent(VideoRecordingStatusArgs.captureEventType.PictureCaptured);
                }
                catch
                {
                    if (_debugLoggingEnabled)
                    { _debugLogger.addLog("Unable to save picture"); }
                }

                lock (_capturingPictureLock)
                { _capturingPicture = false; }
            }
            else
            {
                if (_debugLoggingEnabled)
                { _debugLogger.addLog("Unable to save picture, saving already active"); }
            }

            
        }

        public void stopCapture()
        {
            lock (_videoInitializedLock)
                _videoInitialized = false;

            lock (_threadLock)
            {
                if(_videoRecording == true)
                {
                    _videoRecording = false;
                    vFileWriter.Close();

                    RaiseVideoStatusEvent(VideoRecordingStatusArgs.captureEventType.VideoStopped);
                    if (_debugLoggingEnabled)
                    { _debugLogger.addLog("Stop Capture Successful"); }
                }

              }
        }

        public bool loadVideoDeviceOnStartup(string videoDeviceName, int lastHeight, int lastWidth)
        {
            bool deviceLoadedOk = false;
            string lastVidDevice = "";
            string selectedMonikerString = "";

            //get last video device used from settings file
            try
            {
                lastVidDevice = Properties.Settings.Default.VideoDeviceName;
            }
            catch
            {
                System.Windows.Forms.MessageBox.Show("Could not load last video device name");
            }

            //referesh video devices and execute if some devices are found
            refreshVideoDevices();
            if (VideoDevices.Count > 0)
            {
                //first check if the video device that was used last time is availible
                bool lastDeviceFound = false;
                foreach (FilterInfo vidDevice in VideoDevices)
                {
                    if (lastVidDevice == vidDevice.Name)
                    {
                        lastDeviceFound = true;
                        selectedMonikerString = vidDevice.MonikerString;//v0.0.9
                        VideoDevice = new VideoCaptureDevice(vidDevice.MonikerString);
                        VideoDeviceName = vidDevice.Name;
                        break;
                    }
                }

                //if the last used device was not found, use the first one from the list
                if (lastDeviceFound == false)
                {
                    if (_debugLoggingEnabled)
                    { _debugLogger.addLog("Last Video Device Not found - using " + VideoDevices[0].Name); }
                    VideoDevice = new VideoCaptureDevice(VideoDevices[0].MonikerString);
                    selectedMonikerString = VideoDevices[0].MonikerString;//v0.0.9
                    VideoDeviceName = VideoDevices[0].Name;
                }
                deviceLoadedOk = true;

                //v0.0.9
                _currentCamMoniker = selectedMonikerString;
                setRemoteCapture(_camMonikerStringExclusions);

                int capIndexFound = 0;
                bool resolutionFound = false;
                refreshVideoCapabilites(VideoDevice);
                for (int capIndex = 0; capIndex < _videoCapabilities.Length; capIndex++)
                {
                    if (_videoCapabilities[capIndex].FrameSize.Height == lastHeight 
                        && _videoCapabilities[capIndex].FrameSize.Width == lastWidth)
                    {
                        resolutionFound = true;
                        capIndexFound = capIndex;
                        break;
                    }
                }
                

                if(resolutionFound == true)
                {
                    VideoDevice.VideoResolution = _videoCapabilities[capIndexFound];
                    
                    //remember the video device framsize for Settings Window
                    VideoDeviceWidth = VideoDevice.VideoResolution.FrameSize.Width;
                    VideoDeviceHeight = VideoDevice.VideoResolution.FrameSize.Height;
                }
                else
                {
                    //if last framsize cannot be found, use the first one from the list
                    VideoDevice.VideoResolution = _videoCapabilities[0];

                    //remember the video device framsize for Settings Window
                    VideoDeviceWidth = VideoDevice.VideoResolution.FrameSize.Width;
                    VideoDeviceHeight = VideoDevice.VideoResolution.FrameSize.Height;
                }

                

                //set the a factor for resizing the control
                VidWidthFactor = (double)VideoDevice.VideoResolution.FrameSize.Height / (float)VideoDevice.VideoResolution.FrameSize.Width;

                setupVideoDevice();
            }
            else
            {
                deviceLoadedOk = false;
            }

            return deviceLoadedOk;
        }

        public void setRemoteCapture(string monikerExclCsvString)
        {

            string[] exclusions = monikerExclCsvString.Split(',');
            bool remoteCaptureAllowed = true;

            foreach (var excl in exclusions)
            {
                if (_currentCamMoniker == excl)
                {
                    remoteCaptureAllowed = false;
                    break;
                }
            }

            if(remoteCaptureAllowed)
            {
                if (_debugLoggingEnabled)
                { _debugLogger.addLog("Remote capture is allowed for selected moniker " + _currentCamMoniker); }

                lock (_remCaptAllowedLock)
                { _remCaptAllowed = true; }
            }
            else
            {
                if (_debugLoggingEnabled)
                { _debugLogger.addLog("Remote capture NOT allowed for selected moniker " + _currentCamMoniker); }

                lock (_remCaptAllowedLock)
                { _remCaptAllowed = false; }
            }

        }

        private void setupVideoDevice()
        {
            lock (_videoInitializedLock)
                _videoInitialized = false;

            lock (_threadLock)
            {
                //subscribe to events that come from the video device
                VideoDevice.NewFrame += new NewFrameEventHandler(videoDevice_NewFrame);
                VideoDevice.VideoSourceError += new VideoSourceErrorEventHandler(videoDevice_videoSourceError);
                VideoDevice.PlayingFinished += VideoDevice_PlayingFinished;

                _videoSourcePlayer.VideoSource = VideoDevice;
                _videoSourcePlayer.Start();
            }
        }

        private void VideoDevice_PlayingFinished(object sender, ReasonToFinishPlaying reason)
        {
            bool videoInitialized = false;
            lock (_videoInitializedLock)
                videoInitialized = _videoInitialized;

            bool pause = false;
            lock (_pauseLock)
                pause = _pause;

            lock (_threadLock)
            {

                if (videoInitialized == true && pause == false) 
                {

                    if (recoveryStarted == false)
                    {
                        _vidRecoveryTimer.Change(500, 500);
                        recoveryStarted = true;
                    }
                }
            }
            if(_debugLoggingEnabled)
            { _debugLogger.addLog("VideoDevice Finished Playing --> " + reason.ToString()); }

        }

        private void videoDevice_videoSourceError(object sender, VideoSourceErrorEventArgs eventArgs)
        {
            bool videoInitialized = false;
            lock (_videoInitializedLock)
                videoInitialized = _videoInitialized;
            bool pause = false;
            lock (_pauseLock)
                pause = _pause;

            lock (_threadLock)
            {
                if (videoInitialized == true && pause == false)
                {
                    if (recoveryStarted == false)
                    {
                        _vidRecoveryTimer.Change(500, 500);
                        recoveryStarted = true;
                    }
                }
            }
            if (_debugLoggingEnabled)
            { _debugLogger.addLog("VideoSourceError --> " + eventArgs.Description); }
        }

        void videoDevice_NewFrame(object sender, NewFrameEventArgs eventArgs)
        {
            bool pause = false;
            lock (_pauseLock)
                pause = _pause;

            lock (_currBitmapLock)
            { _currentBitmap = (Bitmap)eventArgs.Frame.Clone(); }

            if (_stopWatchError.IsRunning)
            {
                _stopWatchError.Stop();
                _errorStatus = TimerErrorArgs.errorStatusType.Ok;
                RaiseTimerErrorEvent(TimerErrorArgs.errorStatusType.Ok);
            }
            _stopWatchError.Stop();

            lock(_videoInitializedLock)
            {
                if (_videoInitialized == false)
                    _videoInitialized = true;
            }

            lock (_threadLock)
            {
                if (recoveryStarted)
                {
                    recoveryStarted = false;
                    _vidRecoveryTimer.Change(Timeout.Infinite, Timeout.Infinite);
                    if (_debugLoggingEnabled)
                    { _debugLogger.addLog("Recovery timer in NewFrame Event"); }
                }

                bool captureArmed = false;
                lock (_caputreArmedLock)
                    captureArmed = _captureArmed;

                if (_currStartFrame >= _numOfStartFrames)//v.0.0.8 dont capture until a few frames have passed
                {
                    if (captureArmed == true && pause == false)//v0.0.9 and pause == false
                    {
                        Bitmap frameBitmap = (Bitmap)eventArgs.Frame;
                        capturePicture(frameBitmap);

                        lock (_caputreArmedLock)
                            _captureArmed = false;
                    }
                    if (_videoRecording)
                    {
                        TimeSpan timeSpan = DateTime.Now - _videoCapStartTime;
                        Bitmap videoBitmap = (Bitmap)eventArgs.Frame;
                        try
                        {
                            vFileWriter.WriteVideoFrame(videoBitmap, timeSpan);
                        }
                        catch (Exception ex)
                        {
                            //log errors?
                            if (_debugLoggingEnabled)
                            { _debugLogger.addLog(ex.Message); }
                        }

                    }
                }
                else
                {
                    _currStartFrame++;
                }



            }

        }

        public void refreshVideoCapabilites(VideoCaptureDevice videoDevice)
        {
            //video resolution needs to be added so we know what size to make the video file in videoFileWriter object
            _videoCapabilities = videoDevice.VideoCapabilities;
            
        }
        public void refreshVideoDevices()
        {
            VideoDevices = new FilterInfoCollection(FilterCategory.VideoInputDevice);
        }

        public void Close()
        {
            if (_debugLoggingEnabled)
            { _debugLogger.addLog("Closing Scope View Controller"); }

            SystemEvents.PowerModeChanged -= SystemEvents_PowerModeChanged;

            lock (_pauseLock)
            {
                _pause = true;
            }
            if (_videoSourcePlayer.VideoSource != null)
            {
                //unload events
                VideoDevice.NewFrame -= new NewFrameEventHandler(videoDevice_NewFrame);
                VideoDevice.VideoSourceError -= new VideoSourceErrorEventHandler(videoDevice_videoSourceError);
                VideoDevice.PlayingFinished -= VideoDevice_PlayingFinished;

                //stop video device
                _videoSourcePlayer.SignalToStop();
                _videoSourcePlayer.WaitForStop();
                _videoSourcePlayer.VideoSource = null;
            }
            
        }

    }
}
